#!/usr/bin/env python
#
#  2-run_dynamic_net.py  -  A DXLink test which runs a DX net created
#                           dynamically via DXLSend() calls (still nothing
#                           tricky going on).
#

import sys, os, time, signal, DX

FRAME_BASE   = "FRAME"
FRAME1_FNAME = FRAME_BASE + ".1.tiff"
FRAME2_FNAME = FRAME_BASE + ".2.tiff"

def SigHandler( signum, frame ):
  os.unlink( FRAME1_FNAME )
  os.unlink( FRAME2_FNAME )

def GenerateImages():
  """  Generate a few 24-bit images with DX to use for display.
       They are stored on disk as FRAME.1.tiff and FRAME.2.tiff.
  """

  conn = DX.DXLStartDX( DX.DXEXEC + " -execonly", None )
  DX.exDXLLoadScript( conn, "GenImages.net" )
  DX.DXLSetString   ( conn, "filename_base", FRAME_BASE )
  DX.DXLExecuteOnce ( conn )
  DX.WaitForDXIdle  ( conn )
  DX.DXLExitDX      ( conn )


def DisplayImages():
  """  Display images in a DX Display window.  """

  print "\nPerforming display test:"

  conn = DX.DXLStartDX( DX.DXEXEC + " -execonly", None )

  # Build a few DX script commands
  readimg1 = "img = ReadImage( \"%s\" );" % FRAME1_FNAME
  readimg2 = "img = ReadImage( \"%s\" );" % FRAME2_FNAME
  display  = "Display( img );"

  # Display the images
  DX.DXLSend( conn, readimg1 )
  DX.DXLSend( conn, display )
  DX.WaitForDXIdle( conn )
  DX.DXLSync( conn )                    # Force DX to XSync()

  DX.DXLSend( conn, readimg2 )
  DX.DXLSend( conn, display )
  DX.WaitForDXIdle( conn )
  DX.DXLSync( conn )                    # Force DX to XSync()

  print "  Net execution complete."

  print "  Sleeping for 2 seconds."
  time.sleep(2)

  print "  Terminating DX."
  DX.DXLExitDX( conn )

# Nuke images if killed
signal.signal( signal.SIGHUP , SigHandler )
signal.signal( signal.SIGINT , SigHandler )
signal.signal( signal.SIGQUIT, SigHandler )
signal.signal( signal.SIGTERM, SigHandler )
     
# Generate some images to use for display
GenerateImages()

# Now display them in a DX Display Window
DisplayImages()

# And nuke them
os.unlink( FRAME1_FNAME )
os.unlink( FRAME2_FNAME )

