#!/usr/bin/env python
#
#  3-tk_win.py  -  Now things get a bit more interesting.  Use DXLink to
#                  have DX blast images into a Tk-created X window.
#

import sys, os, time, signal, DX
from DX.DXL import *
from Tkinter import *

IMAGE_WIDTH  = 640
IMAGE_HEIGHT = 480
FRAME_BASE   = "FRAME"
FRAME1_FNAME = FRAME_BASE + ".1.tiff"
FRAME2_FNAME = FRAME_BASE + ".2.tiff"

def SigHandler( signum, frame ):
  os.unlink( FRAME1_FNAME )
  os.unlink( FRAME2_FNAME )

def GenerateImages():
  """  Generate a few 24-bit images with DX to use for display.
       They are stored on disk as FRAME.1.tiff and FRAME.2.tiff.
  """

  conn = DXLStartDX( DX.DXEXEC + " -execonly", None )
  exDXLLoadScript( conn, "GenImages.net" )
  DXLSetString   ( conn, "filename_base", FRAME_BASE )
  DXLExecuteOnce ( conn )
  WaitForDXIdle  ( conn )
  DXLExitDX      ( conn )

def DisplayImages( picture ):
  """  Display images in the window owned by a Tk widget (picture).  """

  print "\nPerforming display test:"

  conn = DXLStartDX( DX.DXEXEC + " -execonly", None )

  # Build a few DX script commands
  readimg1  = "img = ReadImage( \"%s\" );" % FRAME1_FNAME
  readimg2  = "img = ReadImage( \"%s\" );" % FRAME2_FNAME
  where_str = "X%d,,#X%d" % ( picture.winfo_depth(), picture.winfo_id() )
  display   = "Display( img, where=\"%s\" );" % where_str

  # Display the images
  DXLSend( conn, readimg1 )
  DXLSend( conn, display )
  WaitForDXIdle( conn )
  DXLSync( conn )                    # Force DX to XSync()

  DXLSend( conn, readimg2 )
  DXLSend( conn, display )
  WaitForDXIdle( conn )
  DXLSync( conn )                    # Force DX to XSync()

  print "  Net execution complete."

  print "  Sleeping for 2 seconds."
  time.sleep(2)

  print "  Terminating DX."
  DXLExitDX( conn )


if __name__ == "__main__":

  depth = 0

  # Parse arguments

  if len( sys.argv ) > 1:
    try:    depth = int( sys.argv[2] )
    except: depth = 0
    if depth not in [ 8, 24 ] or sys.argv[1] != "-depth":
      print "FORMAT: %s [-depth 8|24]\n" % sys.argv[0]
      sys.exit(0)

  # Nuke images if killed
  signal.signal( signal.SIGHUP , SigHandler )
  signal.signal( signal.SIGINT , SigHandler )
  signal.signal( signal.SIGQUIT, SigHandler )
  signal.signal( signal.SIGTERM, SigHandler )
     
  # Generate some images to use for display
  print "Generating test images."
  GenerateImages()

  # Determine the depth, if non specified
  root     = Tk()
  root.wm_withdraw()

  if not depth: depth = root.winfo_depth()

  if depth == 8: visual = "pseudocolor 8"
  else:          visual = "truecolor  24"

  # Create the GUI
  main     = Toplevel( root )
  frame    = Frame( main, visual=visual, 
                    width = IMAGE_WIDTH, height = IMAGE_HEIGHT )
  frame.pack_propagate( 0 )

  # FIXME:  Hook in a refresh even to make DX redraw
  picture  = Label( frame )
  button   = Button( main, text="Quit", command = main.destroy )

  # main.wm_colormapwindows( picture, frame )

  picture.pack( expand=Y, fill=BOTH )
  frame.pack( expand=N )
  button.pack()

  main.update_idletasks()

  # Now display the images in a Tk X Window
  DisplayImages( picture )

  # And nuke them
  os.unlink( FRAME1_FNAME )
  os.unlink( FRAME2_FNAME )
