#!/usr/bin/env python
#
#  6-msg_hdlr.py  -  Test of message handler callbacks.
#

import sys, os, DX

#----------------------------------------------------------------------------

def MsgHandlerCB( conn, msg, packet_type ):
  """  Handle received DX MessageHandler messages  """

  messages = { DX.PACK_INTERRUPT  : "INTERRUPT ",
               DX.PACK_MACRODEF   : "MACRODEF  ",
               DX.PACK_FOREGROUND : "FOREGROUND",
               DX.PACK_BACKGROUND : "BACKGROUND",
               DX.PACK_ERROR      : "ERROR     ",
               DX.PACK_MESSAGE    : "MESSAGE   ",
               DX.PACK_INFO       : "INFO      ",
               DX.PACK_LINQUIRY   : "LINQUIRY  ",
               DX.PACK_LRESPONSE  : "LRESPONSE ",
               DX.PACK_COMPLETE   : "COMPLETE  ",
               DX.PACK_LINK       : "LINK      " }

  print "<DX Msg> %s:  %s" % ( messages[ packet_type ], msg )

#----------------------------------------------------------------------------

def EnableMessageHandler( conn, enable ):
  #  NOTE: On new connections, a default PACK_ERROR handler is plugged in.
  #        But we can unplug it if we want, or plug something else in instead

  def BuildCB( type ):
    def _( conn, msg, type=type ):
      MsgHandlerCB( conn, msg, type )
    return _

  packetTypes = (
    DX.PACK_INTERRUPT,
    DX.PACK_MACRODEF,
    DX.PACK_FOREGROUND,
    DX.PACK_BACKGROUND,
    DX.PACK_ERROR,
    DX.PACK_MESSAGE,
    DX.PACK_INFO,
    DX.PACK_LINQUIRY,
    DX.PACK_LRESPONSE,
    DX.PACK_COMPLETE,
    DX.PACK_LINK
  )
  
  for i in packetTypes:
    if enable:
      DX.DXLSetMessageHandler( conn, i, None, BuildCB(i) )
    else:
      DX.DXLRemoveMessageHandler( conn, i, None )


#----------------------------------------------------------------------------

def SimpleDXRun():
  
  conn = DX.DXLStartDX( DX.DXEXEC + " -execonly", None )
  DX.DXLSetErrorHandler( conn, lambda conn, msg: None )      # Ignore errors
  EnableMessageHandler  ( conn, 1 )

  DX.DXLSend( conn, "num = 1 + 2;" )
  DX.DXLSend( conn, "DXLOutput( \"result\", num );" )
  DX.DXLExecuteOnce ( conn )
  DX.WaitForDXIdle  ( conn )

  EnableMessageHandler( conn, 0 )
  DX.DXLExitDX      ( conn )

#----------------------------------------------------------------------------

if __name__ == "__main__":

  SimpleDXRun()
