#!/usr/bin/env python
#
#  10-supervise_spin.py  -  An example of what NOT to do.
#
#  NOTES:
#    1) Parameterizing values
#       a) In a UI net, you pop in DXLInputs which can be populated using
#          DXLSet... calls
#       b) In a DXLSend-created network, you simply use placeholder names
#          and re-set these values using DXLSend( 'var=val;' ) calls.
#          See below for an example.
#

import sys, os, math, time, socket, string, DX
from Tkinter import *

IMAGE_WIDTH      = 640
IMAGE_HEIGHT     = 480
HW_RENDER_MODE   = "opengl"                # "none", "gl", "opengl" 
#HW_RENDER_MODE   = "none"                # "none", "gl", "opengl" 
HW_RENDER_APPROX = "none"                 # "none", "box", "dots", "wireframe"

#-----------------------------------------------------------------------------

class Box:
  """  A colored box object displayed in DX.  """
  
  def __init__( self, conn, wgt, hw_render_mode, hw_render_approx ):
    """  Generate a colored box to use for display in DX.

         INPUT  :  conn             - open DX connection
                   wgt              - widget to tell DX to blast in
                   hw_render_mode   - "none", "gl", "opengl"
                   hw_render_approx - "none", "box", "dots", "wireframe"
                   
         RETURNS:  <string> - name of DX script language variable holding obj
    """

    assert hw_render_mode   in [ "none", "gl","opengl" ]
    assert hw_render_approx in [ "none", "box", "dots", "wireframe" ]

    self.conn             = conn
    self.wgt              = wgt
    self.hw_render_mode   = hw_render_mode
    self.hw_render_approx = hw_render_approx

    DX.DXLSend( conn, 'grid = Grid( point=[0,0,0], structure="brick",' +
                           'shape={[1,0,0],[0,1,0],[0,0,1]}, density={2,7,2} );' )
    DX.DXLSend( conn, 'grid2 = Mark( grid, name="positions" );' )
    DX.DXLSend( conn, 'grid3 = Compute( "$0.y", grid2 );' )
    DX.DXLSend( conn, 'grid4 = AutoColor( grid3 );' )
    DX.DXLSend( conn, 'box   = ShowBoundary( grid4 );' )

    if hw_render_mode == "none":
      DX.DXLSend( conn, 'box2  = box;' )
    else:
      DX.DXLSend( conn, 'iact_mode = CollectNamed( 0, "mode" );' )    # 0 = NONE
      
      DX.DXLSend( conn, ('box2  = Options( box, "rendering mode", "hardware", ' +
                      '"rendering approximation", "%s", ' +
                      '"interaction mode", iact_mode );') % 
                     hw_render_approx )

    # Create Supervise objects
    ( width, height, depth ) = ( wgt.winfo_width(), wgt.winfo_height(),
                                 wgt.winfo_depth() )

    # SHOWBOX macro: uses look_to_v and up_v.
    #   We don't make these params because you can't pass parms to the "main"
    #   macro you invoke (exDXLExecuteOnceNamed).
    DX.exDXLBeginMacroDefinition( conn, 'macro SHOWBOX()' )
    DX.DXLSend( conn, ('camera = AutoCamera( box2, resolution=%d, ' +
                    'aspect = %f, direction = look_to_v, ' +
                    'up = up_v );') %
                   ( width, float( height ) / width ) )

    DX.DXLSend( conn, ('where, size, events = SuperviseWindow( "Test", ' +
             'display = NULL, size = [ %d, %d ], offset = NULL, ' +
             'parent = %s, depth = %d );') %
             ( width, height, wgt.winfo_id(), depth ) )

    DX.DXLSend( conn, 'object, camera, where, events = SuperviseState( where, ' +
             'defaultCamera = camera, resetCamera = 1, object = box2, ' +
             'resetObject = NULL, size = size, events = events, ' +
             'mode = NULL, args = NULL );' )

    DX.DXLSend( conn, 'Display( object, camera, where=where );' )
    DX.exDXLEndMacroDefinition( conn )
    
    self.display_macro = 'SHOWBOX'
    self.orient_v      = [ 1,0,0 ]
    self.up_v          = [ 0,1,0 ]

  def SetOrientation( self, orient_v, up_v ):

    self.orient_v = orient_v[:]
    self.up_v     = up_v    [:]

  def Display( self ):

    DX.DXLSend( self.conn, 'look_to_v = [%f,%f,%f];' % tuple(self.orient_v) )
    DX.DXLSend( self.conn, 'up_v      = [%f,%f,%f];' % tuple(self.up_v    ) )

    # NOTE:  We don't call DXLExecuteOnce.  It tries to execute a
    #   "main()" macro, and if it doesn't find one, it calls an error
    #   handler twice, eating socket and CPU bandwidth in two processes.
    #   (not to mention making the error handler useless).
    ##DXLExecuteOnce( self.conn )

    DX.exDXLExecuteOnceNamed( self.conn, self.display_macro )
    DX.WaitForDXIdle( self.conn )
    # DXLSync( self.conn )                    # Force DX to XSync()

#-----------------------------------------------------------------------------

def InitDX( depth, hw_render_mode ):
  """  Initializes and returns a connection to the DX executive.

       INPUT   : hw_render_mode - "none", "gl", "opengl"
                 depth          - 8 or 24; depth of windows to display in
       RETURNS : conn           - an open DX connection

  """

  assert hw_render_mode in [ "none", "gl","opengl" ]

  os.putenv( "DXNO_BACKING_STORE", "1" );     # Make any difference??
  os.putenv( "DXFLING"           , "1" );     # Rotate after release
  
  cmd = DX.DXEXEC + " -execonly"
  if hw_render_mode != "none": cmd = cmd + " -hwrender %s" % hw_render_mode
  if depth          == 8     : cmd = cmd + " -8bitcmap private"
  conn = DX.DXLStartDX( cmd, None )
  #DXLSetMessageDebugging( conn, 1 )

  return conn

#-----------------------------------------------------------------------------

def test():

  depth = 0

  # Parse arguments

  if len( sys.argv ) > 1:
    try:    depth = int( sys.argv[2] )
    except: depth = 0
    if depth not in [ 8, 24 ] or sys.argv[1] != "-depth":
      print "FORMAT: %s [-depth 8|24]\n" % sys.argv[0]
      sys.exit(0)
     
  # Determine the depth, if non specified
  root     = Tk()
  root.wm_withdraw()

  if not depth: depth = root.winfo_depth()

  if depth == 8: visual = "pseudocolor 8"
  else:          visual = "truecolor  24"

  # Create the GUI
  main     = Toplevel( root, visual=visual )
  frame    = Frame( main, visual=visual, 
                    width = IMAGE_WIDTH, height = IMAGE_HEIGHT )
  frame.pack_propagate( 0 )

  picture  = Frame( frame, background="", colormap="new" )
  button   = Button( main, text="Quit", command = root.quit )

  # FIXME:  Need to tell DX to resize when we resize
  picture.pack( expand=Y, fill=BOTH )
  frame.pack( expand=Y, fill=BOTH )
  button.pack()

  main.update_idletasks()

  # Display config
  localhost   = socket.gethostname()
  displayhost = string.split( root.winfo_screen(), ':' )[0]
  if not displayhost or displayhost == "unix":
    displayhost = localhost

  print "\nPYTHON-DX COLOR CUBE ROTATE DEMO"
  print   "________________________________\n"
  print "CONFIGURATION:"
  print "  Local Machine          : %s" % localhost
  print "  Display                : %s" % displayhost
  print "  Hardware Acceleration  : %s" % HW_RENDER_MODE
  print "  Rendering Approximation: %s" % HW_RENDER_APPROX

  # Create a DX connection displaying a color cube inside our Tk widget's window
  conn  = InitDX( depth, HW_RENDER_MODE )
  box   = Box( conn, picture, HW_RENDER_MODE, HW_RENDER_APPROX )

  time1      = time.time()
  num_frames = 0
  
  for i in range(0,360,2):
    box.SetOrientation( [ math.cos( i * math.pi / 180.0 ), 0.5,
                          math.sin( i * math.pi / 180.0 ) ],
                        [0,1,0] )
    box.Display()
    num_frames = num_frames + 1

  time2 = time.time()
  print "Done ... %g frames/sec" % ( num_frames / (time2 - time1) )

  root.mainloop()

  DX.DXLExitDX( conn )

if __name__ == "__main__":
  test()
