#!/usr/bin/env python
#
#  13-userctl_gui.py  -  Give the user control of DX rendering in a DX window
#                        via mouse translations, and provide a GUI to
#                        customize how rendering is performed.
#
#                        This is 09-userctl2.py with a GUI slapped on.
#
#  NOTE:  For reasons we don't yet understand, you must encase the
#         the DXLSend-created visual program inside a macro in order for the
#         software-rendering interactors to work.  The hardware interactors
#         work regardless -- go figure!
#
# 
  
import sys, os, socket, string, DX
from DX.DXServer import *
from Tkinter     import *

IMAGE_WIDTH      = 800
IMAGE_HEIGHT     = 600
HW_RENDER_MODE   = "opengl"                # "none", "gl", "opengl" 
#HW_RENDER_MODE   = "none"                # "none", "gl", "opengl" 
HW_RENDER_APPROX = "none"                 # "none", "box", "dots", "wireframe"

#-----------------------------------------------------------------------------

Interact_modes    = { -1 : "<None>", 0 : "Rotate", 1 : "Pan", 2 : "Zoom" }

Hw_interact_modes = {  1 : "<None>" , 2 : "Rotate"  , 3 : "Cursors", 4 : "Roam",
                       5 : "Pick"   , 6 : "Navigate", 7 : "Zoom"   ,
                       8 : "PanZoom", 9 : "User" }

#-----------------------------------------------------------------------------

class Box:
  """  A colored box object displayed in DX.  """
  
  def __init__( self, dx, wgt ):
    """  Generate a colored box to use for display in DX.

         INPUT  :  dx               - open DXServer object
                   wgt              - widget to tell DX to blast in
                   hw_render_mode   - "none", "gl", "opengl"
                   hw_render_approx - "none", "box", "dots", "wireframe"
                   
         RETURNS:  <string> - name of DX script language variable holding obj
    """

    self.dx               = dx
    self.exec_on_change   = 0
    self.wgt              = wgt

    # BUILD: box = Grid->Mark->Compute->AutoColor->ShowBoundary

    #
    # NOTE:  For reasons we don't yet understand, you must encase the
    #  the DXLSend-created visual program inside a macro in order for the
    #  software-rendering interactors to work.  The hardware interactors
    #  work regardless -- go figure!
    #
    dx.BeginMacroDefinition( 'macro main()' )

    dx.Send( 'grid = Grid( point=[0,0,0], structure="brick",' +
             'shape={[1,0,0],[0,1,0],[0,0,1]}, density={2,7,2} );' )
    dx.Send( 'grid2 = Mark( grid, name="positions" );' )
    dx.Send( 'grid3 = Compute( "$0.y", grid2 );' )
    dx.Send( 'grid4 = AutoColor( grid3 );' )
    dx.Send( 'box   = ShowBoundary( grid4 );' )

    # BUILD: box2 = box->Options

    dx.Send( 'iact_mode = 6;' )
    dx.Send( 'iact_mode_grp = CollectNamed( iact_mode, "mode" );' )

    # If you want to simulate an error, use this line.  DX hates 'string'.
    #dx.Send( "render_mode = '%s'" % HW_RENDER_MODE );
    
    dx.Send( 'box2  = Options( box, "rendering mode", render_mode, '
             '"rendering approximation", hw_render_approx, '
             '"interaction mode", iact_mode_grp );' )

    # Create Supervise objects
    #
    # NOTE:  For HW accel mode, SuperviseState.mode seems to override
    #        Options("interaction mode"), if specified

    # BUILD: Display( box2, camera ) via SuperviseState/SuperviseWindow
    
    depth = wgt.winfo_depth()

    dx.Send( ('camera = AutoCamera( box2, resolution=win_width, ' +
              'aspect = win_aspect, direction = [1,0,0], ' +
              'up = [0,1,0] );') )

    dx.Send( ('where, size, events = SuperviseWindow( "Test", ' +
             'display = NULL, size = win_size, sizeFlag = 1,' +
             'offset = NULL, parent = %s, depth = %d );') %
             ( wgt.winfo_id(), depth ) )

    dx.Send( 'object, camera, where2, events = SuperviseState( where, ' +
             'defaultCamera = camera, resetObject = reset_object, ' +
             'resetCamera = reset_camera, ' +
             'object = box2, size = size, events = events, ' +
             'mode = svwin_iact_mode );' )

    dx.Send( 'Display( object, camera, where=where2 );' )
    dx.EndMacroDefinition()

    # Set up initial defaults 
    if HW_RENDER_MODE == "none": render_mode = "software"
    else:                        render_mode = "hardware"

    dx = self.dx
    self.SetWindowSize                   ( wgt.winfo_width(), 
                                           wgt.winfo_height() )
    self.SetSuperviseStateInteractionMode( -1 )
    self.SetDisplayInteractionMode       ( 6 )
    self.SetRenderMode                   ( render_mode, HW_RENDER_APPROX )

    dx.Send( "reset_object = 1;" )
    dx.Send( "reset_camera = 1;" )
    dx.ExecuteOnce       ()
    dx.Send( "reset_object = 0;" )
    dx.Send( "reset_camera = 0;" )

    dx.ExecuteOnChange()
    self.exec_on_change = 1

  def SetWindowSize( self, width, height ):

    dx = self.dx
    if self.exec_on_change: dx.EndExecuteOnChange()
    dx.Send( "win_width  = %d;" % width  )
    dx.Send( "win_height = %d;" % height )
    
    # In a dx.Send-created net, use dx.Send to set variables instead of
    #   DXLSet* (which is for DXLInput modules).
    # FIXME:  Compute win_size and win_aspect inside DX net
    dx.Send( "win_aspect = %g;" % ( float( height ) / width ) )
    dx.Send( "win_size = [ %d, %d ];" % ( width, height ) )
    dx.Send( "reset_object = 1;" )
    dx.Send( "reset_camera = 1;" )
    if self.exec_on_change: dx.ExecuteOnce       ()
    dx.Send( "reset_object = 0;" )
    dx.Send( "reset_camera = 0;" )
    if self.exec_on_change: dx.ExecuteOnChange   ()
    self.win_size = ( width, height )

  def SetSuperviseStateInteractionMode( self, mode ):
    # VALUES: -1:None, 0:Rotate, 1:Pan, 2:Zoom

    assert mode in [-1,0,1,2]

    dx = self.dx
    if self.exec_on_change: dx.EndExecuteOnChange()
    dx.Send( "svwin_iact_mode = %d;" % mode )
    if self.exec_on_change: dx.ExecuteOnChange   ()
    self.svwin_iact_mode = mode

  def SetDisplayInteractionMode( self, mode ):
    # VALUES: 1:NONE, 2:ROTATE, 3:CURSORS, 4:ROAM, 5:PICK, 6:NAVIGATE, 7:ZOOM,
    #         8:PANZOOM, 9:USER

    assert mode in range(1,10)

    dx = self.dx
    if self.exec_on_change: dx.EndExecuteOnChange()
    dx.Send( "iact_mode    = %d;" % mode )
    dx.Send( "reset_object = 1;" )
    if self.exec_on_change: dx.ExecuteOnce       ()
    dx.Send( "reset_object = 0;" )
    if self.exec_on_change: dx.ExecuteOnChange   ()
    self.iact_mode = mode

  def SetRenderMode( self, mode, hw_render_approx ):

    assert mode in [ "hardware", "software" ]
    assert hw_render_approx in [ "none", "box", "dots", "wireframe" ]
    
    dx = self.dx
    if self.exec_on_change: dx.EndExecuteOnChange()
    dx.Send( 'render_mode      = "%s";' % mode )
    dx.Send( 'hw_render_approx = "%s";' % hw_render_approx )
    dx.Send( 'reset_object     = 1;' )
    if self.exec_on_change: dx.ExecuteOnce       ()
    dx.Send( 'reset_object     = 0;' )
    if self.exec_on_change: dx.ExecuteOnChange   ()
    self.render_mode      = mode
    self.hw_render_approx = hw_render_approx

      
#-----------------------------------------------------------------------------

def InitDX( depth, hw_render_mode ):
  """  Initializes and returns a connection to the DX executive.

       INPUT   : hw_render_mode - "none", "gl", "opengl"
                 depth          - 8 or 24; depth of windows to display in
       RETURNS : dx             - an open DXServer instance

  """

  assert hw_render_mode in [ "none", "gl","opengl" ]

  dx = DXServer()
  dx.SetDXCommand( DX.DXEXEC )
  dx.SetDepth( depth )
  dx.SetHWRenderMode( HW_RENDER_MODE )
  dx.Open()
  
  #dx.SetMessageHandlerOnOff  ( 1 )    # Print all messages (default)
  #dx.SetErrorHandlerOnOff    ( 1 )    # Print errors and exit program (default)
  #dx.SetMessageDebuggingOnOff( 1 )    # DX "stream-of-consciousness" prints

  # A small performance hit, but it aids debugging
  dx.SetSynchronizationOnOff ( 1 )    # Chk msgs & wait idle for every DX call

  return dx

#-----------------------------------------------------------------------------

class App:
  def __init__( self, root, depth, dx ):

    self.root = root

    if depth == 8: visual = "pseudocolor 8"
    else:          visual = "truecolor  24"

    # The top level window
    self.main = main = Toplevel( root, visual=visual )
    self.main.protocol( 'WM_DELETE_WINDOW', sys.exit )

    main.title   ( 'DXL User Control GUI Test' )
    main.iconname( 'UserCtl_GUI' )

    # The menubar
    self.MenubarCreate()

    # Size for the DX display window parent
    self.win_size = ( IMAGE_WIDTH, IMAGE_HEIGHT )

    # The DX display window
    self.box = None
    frame    = Frame( main, visual=visual, 
                      width = self.win_size[0], height = self.win_size[1] )
    frame.pack_propagate( 0 )

    picture  = Frame( frame, background="", colormap="new" )
    picture.pack( expand=Y, fill=BOTH )
    frame.pack( expand=Y, fill=BOTH )

    main.update_idletasks()

    # Set up DX to display in our picture window
    self.box = Box( dx, picture )

    # When the window resizes, tell DX about it
    frame.bind( '<Map>'      , self.__PictureFrameResizeCB )
    frame.bind( '<Configure>', self.__PictureFrameResizeCB )

    # Now set menubar defaults.  These will trigger DX updates as the
    #   box object is updated.
    self.MenubarSetDefaults()    

  def MenubarCreate( self ):

    self.menubar = Frame( self.main, relief=RAISED, borderwidth=2 )
    self.menubar.pack( side=TOP, fill=X )

    # "File" MENU
    mbutton = Menubutton( self.menubar, text="File" );
    mbutton.pack( side=LEFT )
    menu = Menu( mbutton )
    mbutton[ 'menu' ] = menu
    menu.add_command( label="Exit", command=menu.quit )

    # "SV Interactor" MENU
    mbutton = Menubutton( self.menubar, text="SV Interactor" )
    mbutton.pack( side=LEFT )
    menu = Menu( mbutton )
    mbutton[ 'menu' ] = menu

    self.iact_mode_var = IntVar()

    def sort(l): l2 = l[:]; l2.sort(); return l2

    for key in sort( Interact_modes.keys() ):
       menu.add_radiobutton( label    = Interact_modes[ key ],
                             value    = key,
                             variable = self.iact_mode_var,
                             command  = self.__InteractModeCB )

    # "Hardware Interactor" MENU
    mbutton = Menubutton( self.menubar, text="HW Interactor" )
    mbutton.pack( side=LEFT )
    menu = Menu( mbutton )
    mbutton[ 'menu' ] = menu

    self.hw_iact_mode_var = IntVar()

    for key in sort( Hw_interact_modes.keys() ):
       menu.add_radiobutton( label    = Hw_interact_modes[ key ],
                             value    = key,
                             variable = self.hw_iact_mode_var,
                             command  = self.__HWInteractModeCB )

    # "Render Mode" MENU
    mbutton = Menubutton( self.menubar, text="Render Mode" )
    mbutton.pack( side=LEFT )
    menu = Menu( mbutton )
    mbutton[ 'menu' ] = menu

    self.render_mode_var = StringVar()

    for key in [ "software", "hardware" ]:
      menu.add_radiobutton( label    = key,
                            variable = self.render_mode_var,
                            command  = self.__RenderModeCB )

    # "Hardware Render Approximation" MENU
    mbutton = Menubutton( self.menubar, text="HW Render Approx" )
    mbutton.pack( side=LEFT )
    menu = Menu( mbutton )
    mbutton[ 'menu' ] = menu

    self.hw_render_approx_var = StringVar()

    for key in [ "none", "box", "dots", "wireframe" ]:
      menu.add_radiobutton( label    = key,
                            variable = self.hw_render_approx_var,
                            command  = self.__RenderModeCB )

  def MenubarSetDefaults( self ):

    # Defaults: supervise mode = Rotate, hardware interactor = <None>
    self.iact_mode_var.set( -1 )
    self.__InteractModeCB()
    self.hw_iact_mode_var.set( 6 )
    self.__InteractModeCB()

    if HW_RENDER_MODE == "none": mode = "software"
    else:                        mode = "hardware"
    self.render_mode_var.set     ( mode )
    self.hw_render_approx_var.set( HW_RENDER_APPROX )
    self.__RenderModeCB()

  def PanelCreate( self ):

    self.style = Frame( self.main, borderwidth=1, relief=SUNKEN )
    self.style.pack( fill=X )

    self.readout = Label( self.style, text="<This space for rent>" )
    self.readout.pack( side=RIGHT )

  def __PictureFrameResizeCB( self, event ):
    # Store off the width and height to use for box creation, if we haven't
    #   created the box yet.  If we have, pass to DX.
    if self.box:
      self.box.SetWindowSize( event.width, event.height )
    self.win_size = ( event.width, event.height )

  def __InteractModeCB( self ):
    self.box.SetSuperviseStateInteractionMode( self.iact_mode_var.get() )

  def __HWInteractModeCB( self ):
    self.box.SetDisplayInteractionMode( self.hw_iact_mode_var.get() )

  def __RenderModeCB( self ):
    self.box.SetRenderMode( self.render_mode_var.get(),
                            self.hw_render_approx_var.get() )

#-----------------------------------------------------------------------------

def test():

  depth = 0

  # Parse arguments
  if len( sys.argv ) > 1:
    try:    depth = int( sys.argv[2] )
    except: depth = 0
    if depth not in [ 8, 24 ] or sys.argv[1] != "-depth":
      print "FORMAT: %s [-depth 8|24]\n" % sys.argv[0]
      sys.exit(0)

  # Fire up Tk
  root     = Tk()
  root.wm_withdraw()

  # Determine the depth, if non specified
  if not depth: depth = root.winfo_depth()
  
  # Create a server and open a connection
  print "Using hardware render mode = '%s'" % HW_RENDER_MODE
  dx = InitDX( depth, HW_RENDER_MODE )

  # Install a callback to handle DX messages in the Tk event loop
  def DXEventCB( file, mask, cb=dx.GetFileEventCB() ): cb()
  root.tk.createfilehandler( dx.GetSocket(), READABLE, DXEventCB )

  # Create a GUI
  app = App( root, depth, dx )

  # Let DX get to idle (process queued messages, and empty message queue)
  dx.WaitForIdle()
  
  # Process user events
  root.mainloop()

  # Shut down DX
  dx.WaitForIdle()
  root.tk.deletefilehandler( dx.GetSocket() )
  dx.Close()

if __name__ == "__main__":
  test()
