################################################################################
####################                                        ####################
####################        Test the PyServer Class         ####################
####################                                        ####################
################################################################################
#
# Purpose
#    A Python program to test the PyServer class.
# PyServer
#    This Python class creates and manages a Python object that represents
#    OpenDX. Internally, this class calls the OpenDX C library called DXLink.
#    The Python wrappers for DXLink are created using SWIG.
# Author
#    Original: Randall H. Hoper
#    Modifications: Kent Eschenberg eschenbe@psc.edu
# History
#    $Source: /home/eschenbe/CVSvault/DX/py2dx/Install/Test/testServer.py,v $
#    $Author: eschenbe $
#    $Revision: 1.4 $
#    $Date: 2003/08/07 13:31:16 $

import sys, DX, DXServer, Tkinter

# Defaults
depth          = 24
hw_render_mode = "opengl"

# Create a server and open a connection
print "\nFIRING UP DX:\n"
dx = DXServer.DXServer()
dx.SetDXCommand( DX.DXEXEC )
dx.SetDepth( depth )
dx.SetHWRenderMode( hw_render_mode )
dx.Open()
  
dx.SetMessageHandlerOnOff  ( 1 )    # Print all messages (default)
dx.SetSynchronizationOnOff ( 1 )    # Chk msgs & wait idle for every DX call
dx.SetErrorHandlerOnOff    ( 1 )    # Print errors and exit program (default)
dx.SetMessageDebuggingOnOff( 1 )    # DX "stream-of-consciousness" prints

# Fire up Tk
root     = Tkinter.Tk()
root.wm_withdraw()

# Install a callback to handle DX messages in the Tk event loop
def DXEventCB( file, mask, cb=dx.GetFileEventCB() ): cb()
root.tk.createfilehandler( dx.GetSocket(), Tkinter.READABLE, DXEventCB )

# Let DX get to idle (process queued messages, and empty message queue)
dx.WaitForIdle()
  
# Spin in the event loop for half a second
print "\nSleeping for 1/2 second in the event loop.\n"
root.after( 500, root.quit )
root.mainloop()
print "\nDone sleeping..\n"

#---------------------------------------------------------------------------
# SUCCESSFUL EXECUTION TEST
#---------------------------------------------------------------------------
  
print "\nSUCCESS TEST:  Try some useless work, like adding two numbers\n"

# Turn off some of the verbosity now
dx.SetMessageDebuggingOnOff( 0 )
dx.SetMessageHandlerOnOff  ( 0 )

# FIXME consider providing API on DXServer so conn is DXServer object
def DXValueHandler( dx, name, value ):
  print "%s is %s" % ( name, value )

print "Adding 1+2"
dx.BeginMacroDefinition( 'macro main()' )
dx.Send( "num1 = %d + %d;" % (1,2) )
dx.Send( "DXLOutput( \"result\", num1 );" )
dx.EndMacroDefinition()
dx.SetValueHandler( "result", DXValueHandler )
dx.ExecuteOnce()

dx.WaitForIdle()
dx.RemoveValueHandler( "result" )

#---------------------------------------------------------------------------
# FAILURE EXECUTION TEST
#---------------------------------------------------------------------------
  
print "\nFAILURE TEST:  Commit a syntax error via DXLSend()\n"

# Now commit a DX error.
#   Since synchronous mode is on, an exception will be thrown inside this
#   DXLSend call.
try:
  dx.BeginMacroDefinition( 'macro main()' )
  dx.Send( "num1 = 1 + 'abc';" )
  dx.Send( "DXLOutput( \"result\", num1 );" )
  dx.EndMacroDefinition()
  dx.ExecuteOnce()

  # Shut down DX
  dx.WaitForIdle()
  root.tk.deletefilehandler( dx.GetSocket() )
  dx.Close()
except DX.DXError:
  import traceback
  print "\nYep, we got that DXError exception we wanted.  " \
        "Here's the traceback:\n"
  traceback.print_exc( file = sys.stdout )


print "\nTESTING COMPLETE."

#########################  End of testPyServer.py  #############################
